# Copyright (c) 2016 IBM
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.


from shade import exc
from shade.tests.functional import base


class TestRangeSearch(base.BaseFunctionalTestCase):

    def _filter_m1_flavors(self, results):
        """The m1 flavors are the original devstack flavors"""
        new_results = []
        for flavor in results:
            if flavor['name'].startswith("m1."):
                new_results.append(flavor)
        return new_results

    def test_range_search_bad_range(self):
        flavors = self.demo_cloud.list_flavors()
        self.assertRaises(
            exc.OpenStackCloudException,
            self.demo_cloud.range_search, flavors, {"ram": "<1a0"})

    def test_range_search_exact(self):
        flavors = self.demo_cloud.list_flavors()
        result = self.demo_cloud.range_search(flavors, {"ram": "4096"})
        self.assertIsInstance(result, list)
        # should only be 1 m1 flavor with 4096 ram
        result = self._filter_m1_flavors(result)
        self.assertEqual(1, len(result))
        self.assertEqual("m1.medium", result[0]['name'])

    def test_range_search_min(self):
        flavors = self.demo_cloud.list_flavors()
        result = self.demo_cloud.range_search(flavors, {"ram": "MIN"})
        self.assertIsInstance(result, list)
        self.assertEqual(1, len(result))
        # older devstack does not have cirros256
        self.assertTrue(result[0]['name'] in ('cirros256', 'm1.tiny'))

    def test_range_search_max(self):
        flavors = self.demo_cloud.list_flavors()
        result = self.demo_cloud.range_search(flavors, {"ram": "MAX"})
        self.assertIsInstance(result, list)
        self.assertEqual(1, len(result))
        self.assertEqual("m1.xlarge", result[0]['name'])

    def test_range_search_lt(self):
        flavors = self.demo_cloud.list_flavors()
        result = self.demo_cloud.range_search(flavors, {"ram": "<1024"})
        self.assertIsInstance(result, list)
        # should only be 1 m1 flavor with <1024 ram
        result = self._filter_m1_flavors(result)
        self.assertEqual(1, len(result))
        self.assertEqual("m1.tiny", result[0]['name'])

    def test_range_search_gt(self):
        flavors = self.demo_cloud.list_flavors()
        result = self.demo_cloud.range_search(flavors, {"ram": ">4096"})
        self.assertIsInstance(result, list)
        # should only be 2 m1 flavors with >4096 ram
        result = self._filter_m1_flavors(result)
        self.assertEqual(2, len(result))
        flavor_names = [r['name'] for r in result]
        self.assertIn("m1.large", flavor_names)
        self.assertIn("m1.xlarge", flavor_names)

    def test_range_search_le(self):
        flavors = self.demo_cloud.list_flavors()
        result = self.demo_cloud.range_search(flavors, {"ram": "<=4096"})
        self.assertIsInstance(result, list)
        # should only be 3 m1 flavors with <=4096 ram
        result = self._filter_m1_flavors(result)
        self.assertEqual(3, len(result))
        flavor_names = [r['name'] for r in result]
        self.assertIn("m1.tiny", flavor_names)
        self.assertIn("m1.small", flavor_names)
        self.assertIn("m1.medium", flavor_names)

    def test_range_search_ge(self):
        flavors = self.demo_cloud.list_flavors()
        result = self.demo_cloud.range_search(flavors, {"ram": ">=4096"})
        self.assertIsInstance(result, list)
        # should only be 3 m1 flavors with >=4096 ram
        result = self._filter_m1_flavors(result)
        self.assertEqual(3, len(result))
        flavor_names = [r['name'] for r in result]
        self.assertIn("m1.medium", flavor_names)
        self.assertIn("m1.large", flavor_names)
        self.assertIn("m1.xlarge", flavor_names)

    def test_range_search_multi_1(self):
        flavors = self.demo_cloud.list_flavors()
        result = self.demo_cloud.range_search(
            flavors, {"ram": "MIN", "vcpus": "MIN"})
        self.assertIsInstance(result, list)
        self.assertEqual(1, len(result))
        # older devstack does not have cirros256
        self.assertTrue(result[0]['name'] in ('cirros256', 'm1.tiny'))

    def test_range_search_multi_2(self):
        flavors = self.demo_cloud.list_flavors()
        result = self.demo_cloud.range_search(
            flavors, {"ram": "<1024", "vcpus": "MIN"})
        self.assertIsInstance(result, list)
        result = self._filter_m1_flavors(result)
        self.assertEqual(1, len(result))
        flavor_names = [r['name'] for r in result]
        self.assertIn("m1.tiny", flavor_names)

    def test_range_search_multi_3(self):
        flavors = self.demo_cloud.list_flavors()
        result = self.demo_cloud.range_search(
            flavors, {"ram": ">=4096", "vcpus": "<6"})
        self.assertIsInstance(result, list)
        result = self._filter_m1_flavors(result)
        self.assertEqual(2, len(result))
        flavor_names = [r['name'] for r in result]
        self.assertIn("m1.medium", flavor_names)
        self.assertIn("m1.large", flavor_names)

    def test_range_search_multi_4(self):
        flavors = self.demo_cloud.list_flavors()
        result = self.demo_cloud.range_search(
            flavors, {"ram": ">=4096", "vcpus": "MAX"})
        self.assertIsInstance(result, list)
        self.assertEqual(1, len(result))
        # This is the only result that should have max vcpu
        self.assertEqual("m1.xlarge", result[0]['name'])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import mock
import testtools

import shade
from shade._heat import event_utils
from shade._heat import template_utils
from shade import meta
from shade.tests import fakes
from shade.tests.unit import base


class TestStack(base.TestCase):

    @mock.patch.object(shade.OpenStackCloud, 'heat_client')
    def test_list_stacks(self, mock_heat):
        fake_stacks = [
            fakes.FakeStack('001', 'stack1'),
            fakes.FakeStack('002', 'stack2'),
        ]
        mock_heat.stacks.list.return_value = fake_stacks
        stacks = self.cloud.list_stacks()
        mock_heat.stacks.list.assert_called_once_with()
        self.assertEqual(meta.obj_list_to_dict(fake_stacks), stacks)

    @mock.patch.object(shade.OpenStackCloud, 'heat_client')
    def test_list_stacks_exception(self, mock_heat):
        mock_heat.stacks.list.side_effect = Exception()
        with testtools.ExpectedException(
            shade.OpenStackCloudException,
            "Error fetching stack list"
        ):
            self.cloud.list_stacks()

    @mock.patch.object(shade.OpenStackCloud, 'heat_client')
    def test_search_stacks(self, mock_heat):
        fake_stacks = [
            fakes.FakeStack('001', 'stack1'),
            fakes.FakeStack('002', 'stack2'),
        ]
        mock_heat.stacks.list.return_value = fake_stacks
        stacks = self.cloud.search_stacks()
        mock_heat.stacks.list.assert_called_once_with()
        self.assertEqual(meta.obj_list_to_dict(fake_stacks), stacks)

    @mock.patch.object(shade.OpenStackCloud, 'heat_client')
    def test_search_stacks_filters(self, mock_heat):
        fake_stacks = [
            fakes.FakeStack('001', 'stack1', status='GOOD'),
            fakes.FakeStack('002', 'stack2', status='BAD'),
        ]
        mock_heat.stacks.list.return_value = fake_stacks
        filters = {'stack_status': 'GOOD'}
        stacks = self.cloud.search_stacks(filters=filters)
        mock_heat.stacks.list.assert_called_once_with()
        self.assertEqual(meta.obj_list_to_dict(fake_stacks[:1]), stacks)

    @mock.patch.object(shade.OpenStackCloud, 'heat_client')
    def test_search_stacks_exception(self, mock_heat):
        mock_heat.stacks.list.side_effect = Exception()
        with testtools.ExpectedException(
            shade.OpenStackCloudException,
            "Error fetching stack list"
        ):
            self.cloud.search_stacks()

    @mock.patch.object(shade.OpenStackCloud, 'get_stack')
    @mock.patch.object(shade.OpenStackCloud, 'heat_client')
    def test_delete_stack(self, mock_heat, mock_get):
        stack = {'id': 'stack_id', 'name': 'stack_name'}
        mock_get.return_value = stack
        self.assertTrue(self.cloud.delete_stack('stack_name'))
        mock_get.assert_called_once_with('stack_name')
        mock_heat.stacks.delete.assert_called_once_with(stack['id'])

    @mock.patch.object(shade.OpenStackCloud, 'get_stack')
    @mock.patch.object(shade.OpenStackCloud, 'heat_client')
    def test_delete_stack_not_found(self, mock_heat, mock_get):
        mock_get.return_value = None
        self.assertFalse(self.cloud.delete_stack('stack_name'))
        mock_get.assert_called_once_with('stack_name')
        self.assertFalse(mock_heat.stacks.delete.called)

    @mock.patch.object(shade.OpenStackCloud, 'get_stack')
    @mock.patch.object(shade.OpenStackCloud, 'heat_client')
    def test_delete_stack_exception(self, mock_heat, mock_get):
        stack = {'id': 'stack_id', 'name': 'stack_name'}
        mock_get.return_value = stack
        mock_heat.stacks.delete.side_effect = Exception('ouch')
        with testtools.ExpectedException(
            shade.OpenStackCloudException,
            "Failed to delete stack stack_name: ouch"
        ):
            self.cloud.delete_stack('stack_name')

    @mock.patch.object(event_utils, 'poll_for_events')
    @mock.patch.object(shade.OpenStackCloud, 'get_stack')
    @mock.patch.object(shade.OpenStackCloud, 'heat_client')
    def test_delete_stack_wait(self, mock_heat, mock_get, mock_poll):
        stack = {'id': 'stack_id', 'name': 'stack_name'}
        mock_get.side_effect = (stack, None)
        self.assertTrue(self.cloud.delete_stack('stack_name', wait=True))
        mock_heat.stacks.delete.assert_called_once_with(stack['id'])
        self.assertEqual(2, mock_get.call_count)
        self.assertEqual(1, mock_poll.call_count)

    @mock.patch.object(event_utils, 'poll_for_events')
    @mock.patch.object(shade.OpenStackCloud, 'get_stack')
    @mock.patch.object(shade.OpenStackCloud, 'heat_client')
    def test_delete_stack_wait_failed(self, mock_heat, mock_get, mock_poll):
        stack = {'id': 'stack_id', 'name': 'stack_name'}
        stack_failed = {'id': 'stack_id', 'name': 'stack_name',
                        'stack_status': 'DELETE_FAILED',
                        'stack_status_reason': 'ouch'}
        mock_get.side_effect = (stack, stack_failed)
        with testtools.ExpectedException(
            shade.OpenStackCloudException,
            "Failed to delete stack stack_name: ouch"
        ):
            self.cloud.delete_stack('stack_name', wait=True)
        mock_heat.stacks.delete.assert_called_once_with(stack['id'])
        self.assertEqual(2, mock_get.call_count)
        self.assertEqual(1, mock_poll.call_count)

    @mock.patch.object(template_utils, 'get_template_contents')
    @mock.patch.object(shade.OpenStackCloud, 'heat_client')
    def test_create_stack(self, mock_heat, mock_template):
        mock_template.return_value = ({}, {})
        self.cloud.create_stack('stack_name')
        self.assertTrue(mock_template.called)
        mock_heat.stacks.create.assert_called_once_with(
            stack_name='stack_name',
            disable_rollback=False,
            environment={},
            parameters={},
            template={},
            files={},
            timeout_mins=60,
        )

    @mock.patch.object(event_utils, 'poll_for_events')
    @mock.patch.object(template_utils, 'get_template_contents')
    @mock.patch.object(shade.OpenStackCloud, 'get_stack')
    @mock.patch.object(shade.OpenStackCloud, 'heat_client')
    def test_create_stack_wait(self, mock_heat, mock_get, mock_template,
                               mock_poll):
        stack = {'id': 'stack_id', 'name': 'stack_name'}
        mock_template.return_value = ({}, {})
        mock_get.return_value = stack
        ret = self.cloud.create_stack('stack_name', wait=True)
        self.assertTrue(mock_template.called)
        mock_heat.stacks.create.assert_called_once_with(
            stack_name='stack_name',
            disable_rollback=False,
            environment={},
            parameters={},
            template={},
            files={},
            timeout_mins=60,
        )
        self.assertEqual(1, mock_get.call_count)
        self.assertEqual(1, mock_poll.call_count)
        self.assertEqual(stack, ret)

    @mock.patch.object(template_utils, 'get_template_contents')
    @mock.patch.object(shade.OpenStackCloud, 'heat_client')
    def test_update_stack(self, mock_heat, mock_template):
        mock_template.return_value = ({}, {})
        self.cloud.update_stack('stack_name')
        self.assertTrue(mock_template.called)
        mock_heat.stacks.update.assert_called_once_with(
            stack_id='stack_name',
            disable_rollback=False,
            environment={},
            parameters={},
            template={},
            files={},
            timeout_mins=60,
        )

    @mock.patch.object(event_utils, 'poll_for_events')
    @mock.patch.object(template_utils, 'get_template_contents')
    @mock.patch.object(shade.OpenStackCloud, 'get_stack')
    @mock.patch.object(shade.OpenStackCloud, 'heat_client')
    def test_update_stack_wait(self, mock_heat, mock_get, mock_template,
                               mock_poll):
        stack = {'id': 'stack_id', 'name': 'stack_name'}
        mock_template.return_value = ({}, {})
        mock_get.return_value = stack
        ret = self.cloud.update_stack('stack_name', wait=True)
        self.assertTrue(mock_template.called)
        mock_heat.stacks.update.assert_called_once_with(
            stack_id='stack_name',
            disable_rollback=False,
            environment={},
            parameters={},
            template={},
            files={},
            timeout_mins=60,
        )
        self.assertEqual(1, mock_get.call_count)
        self.assertEqual(1, mock_poll.call_count)
        self.assertEqual(stack, ret)

    @mock.patch.object(shade.OpenStackCloud, 'heat_client')
    def test_get_stack(self, mock_heat):
        stack = fakes.FakeStack('azerty', 'stack',)
        mock_heat.stacks.get.return_value = stack
        res = self.cloud.get_stack('stack')
        self.assertIsNotNone(res)
        self.assertEqual(stack.stack_name, res['stack_name'])
        self.assertEqual(stack.stack_name, res['name'])
        self.assertEqual(stack.stack_status, res['stack_status'])



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
import testtools

import os_client_config as occ
import shade
from shade import meta
from shade import _utils
from shade.tests.unit import base
from shade.tests import fakes


RAW_ROLE_ASSIGNMENTS = [
    {
        "links": {"assignment": "http://example"},
        "role": {"id": "123456"},
        "scope": {"domain": {"id": "161718"}},
        "user": {"id": "313233"}
    },
    {
        "links": {"assignment": "http://example"},
        "group": {"id": "101112"},
        "role": {"id": "123456"},
        "scope": {"project": {"id": "456789"}}
    }
]


class TestIdentityRoles(base.TestCase):

    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_list_roles(self, mock_keystone):
        self.op_cloud.list_roles()
        self.assertTrue(mock_keystone.roles.list.called)

    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_get_role(self, mock_keystone):
        role_obj = fakes.FakeRole(id='1234', name='fake_role')
        mock_keystone.roles.list.return_value = [role_obj]

        role = self.op_cloud.get_role('fake_role')

        self.assertTrue(mock_keystone.roles.list.called)
        self.assertIsNotNone(role)
        self.assertEqual('1234', role['id'])
        self.assertEqual('fake_role', role['name'])

    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_create_role(self, mock_keystone):
        role_name = 'tootsie_roll'
        role_obj = fakes.FakeRole(id='1234', name=role_name)
        mock_keystone.roles.create.return_value = role_obj

        role = self.op_cloud.create_role(role_name)

        mock_keystone.roles.create.assert_called_once_with(
            name=role_name
        )
        self.assertIsNotNone(role)
        self.assertEqual(role_name, role['name'])

    @mock.patch.object(shade.OperatorCloud, 'get_role')
    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_delete_role(self, mock_keystone, mock_get):
        role_obj = fakes.FakeRole(id='1234', name='aaa')
        mock_get.return_value = meta.obj_to_dict(role_obj)
        self.assertTrue(self.op_cloud.delete_role('1234'))
        self.assertTrue(mock_keystone.roles.delete.called)

    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_list_role_assignments(self, mock_keystone, mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.role_assignments.list.return_value = RAW_ROLE_ASSIGNMENTS
        ret = self.op_cloud.list_role_assignments()
        mock_keystone.role_assignments.list.assert_called_once_with()
        normalized_assignments = _utils.normalize_role_assignments(
            RAW_ROLE_ASSIGNMENTS
        )
        self.assertEqual(normalized_assignments, ret)

    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_list_role_assignments_filters(self, mock_keystone,
                                           mock_api_version):
        mock_api_version.return_value = '3'
        params = dict(user='123', domain='456', effective=True)
        self.op_cloud.list_role_assignments(filters=params)
        mock_keystone.role_assignments.list.assert_called_once_with(**params)

    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_list_role_assignments_exception(self, mock_keystone,
                                             mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.role_assignments.list.side_effect = Exception()
        with testtools.ExpectedException(
            shade.OpenStackCloudException,
            "Failed to list role assignments"
        ):
            self.op_cloud.list_role_assignments()

    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_list_role_assignments_keystone_v2(self, mock_keystone,
                                               mock_api_version):
        fake_role = fakes.FakeRole(id='1234', name='fake_role')
        mock_api_version.return_value = '2.0'
        mock_keystone.roles.roles_for_user.return_value = [fake_role]
        ret = self.op_cloud.list_role_assignments(
            filters={
                'user': '2222',
                'project': '3333'})
        self.assertEqual(
            ret, [{
                'id': fake_role.id,
                'project': '3333',
                'user': '2222'}])

    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_list_role_assignments_keystone_v2_with_role(self, mock_keystone,
                                                         mock_api_version):
        fake_role1 = fakes.FakeRole(id='1234', name='fake_role')
        fake_role2 = fakes.FakeRole(id='4321', name='fake_role')
        mock_api_version.return_value = '2.0'
        mock_keystone.roles.roles_for_user.return_value = [fake_role1,
                                                           fake_role2]
        ret = self.op_cloud.list_role_assignments(
            filters={
                'role': fake_role1.id,
                'user': '2222',
                'project': '3333'})
        self.assertEqual(
            ret, [{
                'id': fake_role1.id,
                'project': '3333',
                'user': '2222'}])

    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_list_role_assignments_exception_v2(self, mock_keystone,
                                                mock_api_version):
        mock_api_version.return_value = '2.0'
        with testtools.ExpectedException(
            shade.OpenStackCloudException,
            "Must provide project and user for keystone v2"
        ):
            self.op_cloud.list_role_assignments()

    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_list_role_assignments_exception_v2_no_project(self, mock_keystone,
                                                           mock_api_version):
        mock_api_version.return_value = '2.0'
        with testtools.ExpectedException(
            shade.OpenStackCloudException,
            "Must provide project and user for keystone v2"
        ):
            self.op_cloud.list_role_assignments(filters={'user': '12345'})
